<?php

namespace FiloBlu\ProductUrlTools\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Magento\UrlRewrite\Service\V1\Data\UrlRewrite;

/**
 * Class RegenerateProductUrlCommand
 * @package FiloBlu\ProductUrlTools\Console\Command
 */
class RegenerateProductUrlCommand extends Command
{
    /**
     * @var \Magento\UrlRewrite\Model\UrlPersistInterface
     */
    protected $urlPersist;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $collectionFactory;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\Collection
     */
    protected $collection;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resource;

    /**
     * @var \Magento\Catalog\Model\Product\Visibility
     */
    protected $productVisibility;

    /**
     * @var \Magento\Catalog\Model\Product\Attribute\Source\Status
     */
    protected $productStatus;

    /**
     * RegenerateProductUrlCommand constructor.
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $collectionFactory
     * @param \Magento\UrlRewrite\Model\UrlPersistInterface $urlPersist
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus
     * @param \Magento\Catalog\Model\Product\Visibility $productVisibility
     * @param \Magento\Framework\App\ResourceConnection $resource
     * @param null $name
     */
    public function __construct(
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory  $collectionFactory,
        \Magento\UrlRewrite\Model\UrlPersistInterface $urlPersist,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Catalog\Model\Product\Attribute\Source\Status $productStatus,
        \Magento\Catalog\Model\Product\Visibility $productVisibility,
        \Magento\Framework\App\ResourceConnection $resource,
        $name = null
    )
    {
        $this->collectionFactory = $collectionFactory;
        $this->urlPersist = $urlPersist;
        $this->logger = $logger;
        $this->productVisibility = $productVisibility;
        $this->productStatus = $productStatus;
        $this->resource = $resource;
        parent::__construct($name);
    }

    /**
     * @param InputInterface $inp
     * @param OutputInterface $out
     * @return int|void|null
     */
    public function execute(InputInterface $inp, OutputInterface $out)
    {
        $this->collection = $this->collectionFactory->create();
        $store_id = $inp->getOption('store');
        $this->collection->addStoreFilter($store_id)->setStoreId($store_id);
        $pids = $inp->getArgument('pids');
        $allStores = $this->getAllStoreIds();
        $storesList = [];

        if (!empty($pids)) {
            $this->collection->addIdFilter($pids);
        }
        else {
            $this->collection->addAttributeToFilter('status', ['in' => $this->productStatus->getVisibleStatusIds()]);
            $this->collection->setVisibility($this->productVisibility->getVisibleInSiteIds());
        }

        $this->collection->addAttributeToSelect(['url_path', 'url_key']);


        if (!empty($store_id)) {
            if (isset($allStores[$store_id])) {
                $storesList = array(
                    $store_id => $allStores[$store_id]
                );
            }
        } else {
            $storesList = $allStores;
        }

        foreach ($storesList as $store_id) {
            $this->collection->clear();
            $this->collection->setStore($store_id);
            $list = $this->collection->load();

            foreach ($list as $product) {

                $product->setStoreId($store_id);

                $this->urlPersist->deleteByData([
                    UrlRewrite::ENTITY_ID => $product->getId(),
                    UrlRewrite::ENTITY_TYPE => ProductUrlRewriteGenerator::ENTITY_TYPE,
                    UrlRewrite::REDIRECT_TYPE => 0,
                    UrlRewrite::STORE_ID => $product->getStoreId()
                ]);

                try {

                    $urls = $this->productUrlRewriteGenerator->generate($product);
                    $this->urlPersist->replace($urls);
                    $this->logger->info('processed url for product with id : ' . $product->getId() . " storeID " . $store_id);

                } catch (\Exception $e) {
                    $out->writeln('<error>Duplicated url for ' . $product->getId() . ' ' . $e->getMessage() . ' </error>');
                }
            }
        }
    }

    /**
     * @return array
     */
    public function getAllStoreIds()
    {
        $result = [];

        $sql = $this->resource->select()
            ->from($this->resource->getTableName('store'), array('store_id', 'code'))
            ->where('`code` <> ?', 'admin')
            ->order('store_id', 'ASC');

        $queryResult = $this->resource->fetchAll($sql);

        foreach ($queryResult as $row) {
            if (isset($row['store_id']) && (int)$row['store_id'] > 0) {
                $result[(int)$row['store_id']] = $row['store_id'];
            }
        }

        return $result;
    }

    /**
     *
     */
    protected function configure()
    {
        $this->setName('tools:regenerateurl')
            ->setDescription('Regenerate url for given products, recommended launch regenerate url key first')
            ->addArgument(
                'pids',
                InputArgument::IS_ARRAY,
                'Products to regenerate'
            )
            ->addOption(
                'store', 's',
                InputOption::VALUE_REQUIRED,
                'Use the specific Store View'
            );
    }
}
